--Automates handling of tourney sets
--By Onyo

rawset(_G, "aTourney", {}) -- ATO global namespace, see exposed value descriptions below
aTourney.tourneyInSet = false -- Bool. If true, a tourney set is currently ongoing.
aTourney.tourneyMapList = {} -- List of upcoming maps (numerical IDs) in a defined set. Keep in mind these get removed one by one as the set progresses.
aTourney.isRandomSet = false -- Bool. If true, the set currently being played is random and will not have map data.
aTourney.raceNumber = 0 -- Int. The current race number in the set. This doesn't increment until intermission starts.
aTourney.totalMaps = 0 -- Int. The number of maps in the set. Should be set up and ready to read when a tourney set begins.
aTourney.bluetotalscore = 0 -- Int. Friendmod specific. Holds the total score for the entire tourney set for the blue team.
aTourney.orangetotalscore = 0 -- Int. Same as above but orange.

--          Parameters
--Toggles pausing at the end of the last map in a set
local PauseOnSetEnd = CV_RegisterVar({
	name = "ato_pausingonsetend",
	defaultvalue = 1,
	flags = CV_NETVAR,
	PossibleValue = CV_OnOff,
})
--Toggles printing spectator's scores to log at end of every round
local LogSpectatorScores = CV_RegisterVar({
	name = "ato_logspectatorscores",
	defaultvalue = 0,
	flags = CV_NETVAR,
	PossibleValue = CV_OnOff,
})

--          Script vars
local didAnnounce = false
local clearscoreflag = false
local RS_THINK, RS_VOTE, RS_INT = 1, 2, 3
local roundstatus = RS_THINK
local tIntCounter = 20 * TICRATE --Placeholder, this script is loaded before inttime is set in a server's config file
local logMapBuffer = ""
local savedScores = {}
local lastRaceOfRandom = false
local didCommand = false

--Team color reference for score reports
--Expanded to include custom client colors - Onyo
local colLookup = { -- thanks ampersand
	[SKINCOLOR_WHITE] = "\x80",
	[SKINCOLOR_SILVER] = "\x80",
	[77] = "\x80",
	[101] = "\x80",
	[104] = "\x80",
	[131] = "\x80",
	[184] = "\x80",
	[189] = "\x80",
	[90] = "\x81",
	[93] = "\x81",
	[94] = "\x81",
	[125] = "\x81",
	[126] = "\x81",
	[127] = "\x81",
	[197] = "\x81",
	[198] = "\x81",
	[200] = "\x81",
	[203] = "\x81",
	[SKINCOLOR_POPCORN] = "\x82",
	[SKINCOLOR_QUARRY] = "\x82",
	[SKINCOLOR_YELLOW] = "\x82",
	[SKINCOLOR_MUSTARD] = "\x82",
	[SKINCOLOR_CROCODILE] = "\x82",
	[SKINCOLOR_OLIVE] = "\x82",
	[114] = "\x82",
	[154] = "\x82",
	[185] = "\x82",
	[SKINCOLOR_LIME] = "\x83",
	[SKINCOLOR_HANDHELD] = "\x83",
	[SKINCOLOR_MOSS] = "\x83",
	[SKINCOLOR_CAMOUFLAGE] = "\x83",
	[SKINCOLOR_ROBOHOOD] = "\x83",
	[SKINCOLOR_MINT] = "\x83",
	[SKINCOLOR_GREEN] = "\x83",
	[SKINCOLOR_PINETREE] = "\x83",
	[67] = "\x83",
	[66] = "\x83",
	[SKINCOLOR_EMERALD] = "\x83",
	[SKINCOLOR_SWAMP] = "\x83",
	[SKINCOLOR_DREAM] = "\x83",
	[119] = "\x83",
	[155] = "\x83",
	[156] = "\x83",
	[159] = "\x83",
	[160] = "\x83",
	[162] = "\x83",
	[164] = "\x83",
	[165] = "\x83",
	[166] = "\x83",
	[168] = "\x83",
	[169] = "\x83",
	[179] = "\x83",
	[84] = "\x84",
	[85] = "\x84",
	[86] = "\x84",
	[87] = "\x84",
	[123] = "\x84",
	[124] = "\x84",
	[172] = "\x84",
	[174] = "\x84",
	[180] = "\x84",
	[181] = "\x84",
	[188] = "\x84",
	[191] = "\x84",
	[SKINCOLOR_CINNAMON] = "\x85",
	[SKINCOLOR_RUBY] = "\x85",
	[SKINCOLOR_RASPBERRY] = "\x85",
	[SKINCOLOR_CHERRY] = "\x85",
	[SKINCOLOR_RED] = "\x85",
	[SKINCOLOR_CRIMSON] = "\x85",
	[SKINCOLOR_MAROON] = "\x85",
	[SKINCOLOR_FLAME] = "\x85",
	[SKINCOLOR_SCARLET] = "\x85",
	[SKINCOLOR_KETCHUP] = "\x85",
	[137] = "\x85",
	[138] = "\x85",
	[139] = "\x85",
	[140] = "\x85",
	[202] = "\x85",
	[SKINCOLOR_GREY] = "\x86",
	[SKINCOLOR_NICKEL] = "\x86",
	[SKINCOLOR_BLACK] = "\x86",
	[SKINCOLOR_SKUNK] = "\x86",
	[SKINCOLOR_JET] = "\x86",
	[102] = "\x86",
	[103] = "\x86",
	[132] = "\x86",
	[SKINCOLOR_DAWN] = "\x87",
	[SKINCOLOR_SUNSET] = "\x87",
	[SKINCOLOR_CREAMSICLE] = "\x87",
	[SKINCOLOR_ORANGE] = "\x87",
	[SKINCOLOR_PUMPKIN] = "\x87",
	[SKINCOLOR_ROSEWOOD] = "\x87",
	[SKINCOLOR_BURGUNDY] = "\x87",
	[SKINCOLOR_TANGERINE] = "\x87",
	[112] = "\x87",
	[141] = "\x87",
	[149] = "\x87",
	[68] = "\x88",
	[69] = "\x88",
	[70] = "\x88",
	[71] = "\x88",
	[72] = "\x88",
	[73] = "\x88",
	[74] = "\x88",
	[75] = "\x88",
	[83] = "\x88",
	[120] = "\x88",
	[121] = "\x88",
	[122] = "\x88",
	[167] = "\x88",
	[171] = "\x88",
	[173] = "\x88",
	[175] = "\x88",
	[176] = "\x88",
	[177] = "\x88",
	[178] = "\x88",
	[192] = "\x88",
	[88] = "\x89",
	[89] = "\x89",
	[91] = "\x89",
	[95] = "\x89",
	[96] = "\x89",
	[97] = "\x89",
	[98] = "\x89",
	[99] = "\x89",
	[128] = "\x89",
	[147] = "\x89",
	[193] = "\x89",
	[194] = "\x89",
	[195] = "\x89",
	[SKINCOLOR_GOLD] = "\x8A",
	[SKINCOLOR_ROYAL] = "\x8A",
	[SKINCOLOR_BRONZE] = "\x8A",
	[SKINCOLOR_COPPER] = "\x8A",
	[79] = "\x8A",
	[111] = "\x8A",
	[113] = "\x8A",
	[142] = "\x8A",
	[143] = "\x8A",
	[144] = "\x8A",
	[153] = "\x8A",
	[SKINCOLOR_ARTICHOKE] = "\x8B",
	[SKINCOLOR_VOMIT] = "\x8B",
	[SKINCOLOR_GARDEN] = "\x8B",
	[SKINCOLOR_TEA] = "\x8B",
	[SKINCOLOR_PISTACHIO] = "\x8B",
	[115] = "\x8B",
	[116] = "\x8B",
	[117] = "\x8B",
	[118] = "\x8B",
	[150] = "\x8B",
	[157] = "\x8B",
	[158] = "\x8B",
	[161] = "\x8B",
	[163] = "\x8B",
	[170] = "\x8B",
	[SKINCOLOR_PIGEON] = "\x8C",
	[76] = "\x8C",
	[78] = "\x8C",
	[105] = "\x8C",
	[106] = "\x8C",
	[182] = "\x8C",
	[186] = "\x8C",
	[187] = "\x8C",
	[190] = "\x8C",
	[196] = "\x8C",
	[204] = "\x8C",
	[SKINCOLOR_FAIRY] = "\x8D",
	[SKINCOLOR_SALMON] = "\x8D",
	[SKINCOLOR_PINK] = "\x8D",
	[SKINCOLOR_ROSE] = "\x8D",
	[SKINCOLOR_BRICK] = "\x8D",
	[SKINCOLOR_LEMONADE] = "\x8D",
	[92] = "\x8D",
	[100] = "\x8D",
	[129] = "\x8D",
	[130] = "\x8D",
	[134] = "\x8D",
	[135] = "\x8D",
	[136] = "\x8D",
	[145] = "\x8D",
	[146] = "\x8D",
	[199] = "\x8D",
	[201] = "\x8D",
	[SKINCOLOR_SEPIA] = "\x8E",	
	[SKINCOLOR_BEIGE] = "\x8E",	
	[SKINCOLOR_WALNUT] = "\x8E",
	[SKINCOLOR_BROWN] = "\x8E",	
	[SKINCOLOR_LEATHER] = "\x8E",
	[80] = "\x8E",
	[81] = "\x8E",
	[107] = "\x8E",
	[108] = "\x8E",
	[133] = "\x8E",
	[148] = "\x8E",
	[151] = "\x8E",
	[152] = "\x8E",
	[SKINCOLOR_PEACH] = "\x8F",
	[SKINCOLOR_CARAMEL] = "\x8F",
	[SKINCOLOR_CREAM] = "\x8F",
	[109] = "\x8F",
	[110] = "\x8F",
	[183] = "\x8F",
}

--          General use functions

local function blueprefix() -- SHUT UP SHUT UP SHUT UP
	if CV_FindVar("fr_bluecolorlocal") and CV_FindVar("fr_bluecolorlocal") .value
		return colLookup[CV_FindVar("fr_bluecolorlocal") .value]
	end
	if CV_FindVar("fr_bluecolor") then
		return colLookup[CV_FindVar("fr_bluecolor") .value]
	end
		return "\x84"
end

local function orangeprefix() -- NIGHTMARE NIGHTMARE NIGHTMARE NIGHTMARE
	if CV_FindVar("fr_orangecolorlocal") and CV_FindVar("fr_orangecolorlocal") .value
		return colLookup[CV_FindVar("fr_orangecolorlocal") .value]
	end
	if CV_FindVar("fr_orangecolor") then
		return colLookup[CV_FindVar("fr_orangecolor") .value]
	end
		return "\x87"
end

--Converts MAPXX format into full integer values
local function ConvertMapID(mapID)
	--Just in case scrubbing
	mapID = string.gsub(tostring(mapID), "map", "")
	mapID = string.gsub(tostring(mapID), "MAP", "")
	--If there's no letters in the ID then we don't need to do anything
	if tonumber(mapID) ~= nil then
		return mapID
	end
	--Too long/short to be a valid map ID at this point
	if #mapID > 2 or #mapID < 2 then
		return -1
	end
	
	--Char in C++ converts to int, lua doesn't have that luxury, so...
	local conDig = {
		['a'] = 0, ['A'] = 0,
		['b'] = 1, ['B'] = 1,
		['c'] = 2, ['C'] = 2,
		['d'] = 3, ['D'] = 3,
		['e'] = 4, ['E'] = 4,
		['f'] = 5, ['F'] = 5,
		['g'] = 6, ['G'] = 6,
		['h'] = 7, ['H'] = 7,
		['i'] = 8, ['I'] = 8,
		['j'] = 9, ['J'] = 9,
		['k'] = 10, ['K'] = 10,
		['l'] = 11, ['L'] = 11,
		['m'] = 12, ['M'] = 12,
		['n'] = 13, ['N'] = 13,
		['o'] = 14, ['O'] = 14,
		['p'] = 15, ['P'] = 15,
		['q'] = 16, ['Q'] = 16,
		['r'] = 17, ['R'] = 17,
		['s'] = 18, ['S'] = 18,
		['t'] = 19, ['T'] = 19,
		['u'] = 20, ['U'] = 20,
		['v'] = 21, ['V'] = 21,
		['w'] = 22, ['W'] = 22,
		['x'] = 23, ['X'] = 23,
		['y'] = 24, ['Y'] = 24,
		['z'] = 25, ['Z'] = 25,
	}
	--Letters in digit 2 are conDig[] + 10
	local Digit1 = 0
	local Digit2 = 0
	for i = 1, #mapID do
		local c = mapID:sub(i, i)
		if tonumber(c) ~= nil then
			if i == 1 then
				Digit1 = c
			else
				Digit2 = c
			end
		else
			if conDig[tostring(c)] == nil then
				--Invalid character
				return -1
			end
			if i == 1 then
				Digit1 = conDig[tostring(c)]
			else
				Digit2 = conDig[tostring(c)] + 10
			end
		end
	end
	return ((36 * Digit1 + Digit2) + 100)
end

--Verify that the inputted map set has valid IDs, and the map actually exists before starting
local function VerifyMapSet(player, inputTable)
	if inputTable == nil then
		CONS_Printf(player, "\nNo map ID found! Enter Map IDs, seperated by spaces.")
		CONS_Printf(player, "\nEX: ATO_StartSet map24 mapa3 map21 map1")
		return false
	end
	--Assemble table of maps
	local mapList = {}
	local count = 0
	for k, v in ipairs(inputTable) do
		local holder = ConvertMapID(v)
		if holder != -1 then
			table.insert(mapList, holder)
			count = $ + 1
		else
			CONS_Printf(player, tostring(v).." is an invalid MapID")
			return false
		end
	end	
	--Verify that each item is selecting a valid map
	for _, mapID in ipairs(mapList) do
		if mapheaderinfo[tostring(mapID)] == nil then
			CONS_Printf(player, "\nMap ID "..tostring(mapID).." was not found. Set not started.")
			return false
		end
	end
	--Load the validated data and begin the set
	aTourney.tourneyMapList = mapList
	aTourney.totalMaps = count
	aTourney.raceNumber = 0
	aTourney.tourneyInSet = true
	didAnnounce = false
	didCommand = false
	tIntCounter = CV_FindVar("inttime").value * TICRATE
	aTourney.bluetotalscore = 0
	aTourney.orangetotalscore = 0
	return true
end

--Sorting function
local function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

local function PrintScores()
	--Some Elim/Friendmod code is here from Freeman
	local stuffToSort = {}
	local playerObjects = {}
	local sortedplayers = {}
	local blueTeam = "Blue"
	local orangeTeam = "Orange"
	if CV_FindVar("fr_enabled") and CV_FindVar("fr_enabled").value then
		--I can only support manually set team names here as the random names aren't exposed in Friendmod
		if CV_FindVar("fr_bluename").string ~= "Random" then
			blueTeam = CV_FindVar("fr_bluename").string
		end
		if CV_FindVar("fr_orangename").string ~= "Random" then
			orangeTeam = CV_FindVar("fr_orangename").string
		end
	end
	--ignore spectators
	for p in players.iterate do
		if p.valid then
			if not LogSpectatorScores.value and p.mo ~= nil and p.mo.valid then
				stuffToSort[p.name] = p.score
				playerObjects[p.name] = p
				table.insert(sortedplayers, p)
			elseif LogSpectatorScores.value then
				stuffToSort[p.name] = p.score
				playerObjects[p.name] = p
				table.insert(sortedplayers, p)
			end				
		end
	end
	if logMapBuffer then --..just in case. Scores are important.
		print("\x85**SCORE PRINTOUT** \x83Race Number "..tostring(aTourney.raceNumber).." \x87"..logMapBuffer)
		if CV_FindVar("fr_enabled") and CV_FindVar("fr_enabled").value then
			print("TEAM "..blueprefix()..blueTeam.."\128 VS TEAM "..orangeprefix()..orangeTeam)
		end
		if CV_FindVar("elimination") and CV_FindVar("elimination").value then
			print("\131##\128| \131NAME                 \128| \131PTS \128|\133DMG \128|\135CMB \128|\131KIL")
		else
			print("\131##\128| \131NAME                 \128| \131PTS")
		end	
	end
	if CV_FindVar("fr_enabled") and CV_FindVar("fr_enabled").value then
		table.sort(playerObjects,function(a, b)
			return a.personalscore > b.personalscore
		end)
		local i = 0
		local bluetotalthisround = 0
		local orangetotalthisround = 0
		for _, q in ipairs(sortedplayers)
			i=$+1
			local pos = "??"
			local pname = "WHO THE FUCK??"
			local pscore = "##??" // I dont care if this actually ever shows up but it will be funny if it does
			if q.FRteam == 1 then
				pos = string.format("%02i",i)
				pname = blueprefix()..string.format("%-20s",q.name)
				pscore = blueprefix()..string.format("%4s",q.personalscore)
				bluetotalthisround = $+q.personalscore
			else
				pos = string.format("%02i",i)
				pname = orangeprefix()..string.format("%-20s",q.name)
				pscore = orangeprefix()..string.format("%4s",q.personalscore)
				orangetotalthisround = $+q.personalscore
			end
			print("\130"..pos.."\128| "..pname.." |\132"..pscore)
		end
		print(string.rep("-",31))
		print("THIS ROUND: "..blueprefix()..bluetotalthisround.."\128 - "..orangeprefix()..orangetotalthisround)
		aTourney.bluetotalscore = $+bluetotalthisround
		aTourney.orangetotalscore = $+orangetotalthisround
		print("GRAND TOTAL: "..blueprefix()..aTourney.bluetotalscore.."\128 - "..orangeprefix()..aTourney.orangetotalscore)
	else
		local posCounter = 1
		local lastScore = -1
		local tiePosTracker = -1
		for k,v in spairs(stuffToSort, function(t,a,b) return t[a] > t[b] end) do
			local posString = tostring(posCounter)
			if lastScore == v then
				--This player is tied with the last one
				posString = tostring(tiePosTracker)
			else
				tiePosTracker = posCounter
			end
			lastScore = v
			--Empty space around name length - 25
			local padPointer = string.len(k)
			local namepadding = ""
			while (padPointer < 21)
			do
				namepadding = $.." "
				padPointer = $ + 1
			end
			padPointer = string.len(v)
			local scorepadding = ""
			while (padPointer < 4)
			do
				scorepadding = $.." "
				padPointer = $ + 1
			end
			if string.len(posString) == 1 then
				posString = " "..$
			end
			--Elimination specific data
			local elimString = ""
			if playerObjects[k] and playerObjects[k].hpmod then
				elimString = "\x80| \133"..playerObjects[k].hpmod.dmgpoints.."\x80| \135"..playerObjects[k].hpmod.combopoints.."\x80| \131"..playerObjects[k].hpmod.killpoints		
			end
			print("\x82"..tostring(posString).."\x80| "..k..namepadding.."|\x84"..scorepadding..tostring(v)..elimString)
			posCounter = $ + 1
		end
	end
	print("\x80______________________________")
end

--          Commands
--I wonder what this does
local function ClearScores(player)
	if (IsPlayerAdmin(player) or player == server) then
		COM_BufInsertText(server, "clearscores")
		chatprint("\133<TOURNEY>\x83 Scores have been reset", true)
	else
		CONS_Printf(player, "\nYou must be an admin to use this command.")	
		return
	end
end
COM_AddCommand("ATO_ClearScores", ClearScores)

local function ClearSet(player)
	if (IsPlayerAdmin(player) or player == server) and aTourney.tourneyInSet then
		aTourney.tourneyMapList = {}
		aTourney.tourneyInSet = false
		didAnnounce = false
		didCommand = false
		savedScores = {}
		tIntCounter = CV_FindVar("inttime").value * TICRATE
		aTourney.isRandomSet = false
		aTourney.bluetotalscore = 0
		aTourney.orangetotalscore = 0
		lastRaceOfRandom = false
		COM_BufInsertText(server, "csay Tourney set has been cancelled")
		CONS_Printf(player, "\nTourney set has been cancelled")	
	else
		CONS_Printf(player, "\nThere is no active set or you are not an admin.")	
	end
end
COM_AddCommand("ATO_ClearSet", ClearSet)

local function ShowSetMaps(player)
	if aTourney.tourneyInSet and not aTourney.isRandomSet then
		CONS_Printf(player, "Upcoming maps, in order:")
		for k, v in ipairs(aTourney.tourneyMapList) do
			--These were already vetted, so we know they exist.
			--R-right?
			CONS_Printf(player, mapheaderinfo[tostring(v)].lvlttl.." "..mapheaderinfo[tostring(v)].zonttl)
		end
	elseif aTourney.isRandomSet then
		CONS_Printf(player, "Set is running with random/voted maps.")
	else
		CONS_Printf(player, "A set is not running.")
	end
end
COM_AddCommand("ATO_ShowSetMaps", ShowSetMaps)

local function StartSet(player, ...)
	if (IsPlayerAdmin(player) or player == server) then
		if aTourney.tourneyInSet then
			CONS_Printf(player, "\nA tourney is already running. Use ATO_ClearSet to stop it.")	
			return
		end
		if CV_FindVar("advancemap") and CV_FindVar("advancemap").value ~= 3 then
			CONS_Printf(player, "\nSet advancemap 3 before runnining this command.")
			return
		end
		local checkTable = {...}
		if VerifyMapSet(player, checkTable) then
			clearscoreflag = true
			aTourney.isRandomSet = false
			ShowSetMaps(player)			
			
			chatprint("\133<TOURNEY>\x83 A set is starting!", true)
			chatprint("\133<TOURNEY>\x83 Set now running for ".. tostring(aTourney.totalMaps).." maps!", true)
			if roundstatus == RS_THINK then
				COM_BufInsertText(server, "exitlevel")
			elseif roundstatus == RS_VOTE then --Don't do this unless you've told players it's starting, because it's going to start immediately
				--Move maps NOW
				aTourney.raceNumber = 1
				chatprint("\133<TOURNEY>\x83 Up Now: "..mapheaderinfo[tostring(aTourney.tourneyMapList[1])].lvlttl.." "..mapheaderinfo[tostring(aTourney.tourneyMapList[1])].zonttl, true)
				if aTourney.raceNumber >= aTourney.totalMaps then
					chatprint("\133<TOURNEY>\x85 LAST RACE OF THE SET!", true) --Bro, you made a set with one map?
				else
					chatprint("\133<TOURNEY>\x83 Race #"..tostring(aTourney.raceNumber).." of "..tostring(aTourney.totalMaps), true)	
				end
				COM_BufInsertText(server, "map "..tostring(aTourney.tourneyMapList[1]))
			end
			
		end		
	else
		CONS_Printf(player, "\nYou must be an admin to use this command.")	
	end
end
COM_AddCommand("ATO_StartSet", StartSet)

local function StartRandomSet(player, count)
	if (IsPlayerAdmin(player) or player == server) then
		if aTourney.tourneyInSet then
			CONS_Printf(player, "\nA tourney is already running. Use ATO_ClearSet to stop it.")	
			return
		end
		if roundstatus == RS_VOTE then
			CONS_Printf(player, "\nCan't run this on the vote screen - ATO doesn't handle random map changing so your first map will not be random!")	
			return	
		end
		if count == nil or tonumber(count) == nil then
			CONS_Printf(player, "\nUsage: ATO_StartRandomSet [Number of races]")	
			return	
		end
		
		--All checks passed
		aTourney.totalMaps = tonumber(count)
		aTourney.raceNumber = 0
		aTourney.tourneyInSet = true
		didAnnounce = false
		didCommand = false
		tIntCounter = CV_FindVar("inttime").value * TICRATE
		aTourney.isRandomSet = true
		clearscoreflag = true
		aTourney.bluetotalscore = 0
		aTourney.orangetotalscore = 0
		
		chatprint("\133<TOURNEY>\x83 A set is starting!", true)
		chatprint("\133<TOURNEY>\x83 Set now running for ".. tostring(aTourney.totalMaps).." maps!", true)
		if roundstatus == RS_THINK then
			COM_BufInsertText(server, "exitlevel")
		end
	else
		CONS_Printf(player, "\nYou must be an admin to use this command.")	
	end
end
COM_AddCommand("ATO_StartRandomSet", StartRandomSet)

--          Hooks

local function ThinkFrame()
	if aTourney.tourneyInSet and roundstatus != RS_THINK then
		--Progress the tourney
		didAnnounce = false
		didCommand = false
		lastRaceOfRandom = false
		if not aTourney.isRandomSet then
			logMapBuffer = mapheaderinfo[tostring(aTourney.tourneyMapList[1])].lvlttl.." "..mapheaderinfo[tostring(aTourney.tourneyMapList[1])].zonttl
			table.remove(aTourney.tourneyMapList, 1)
		elseif mapheaderinfo[gamemap]
			logMapBuffer = mapheaderinfo[gamemap].lvlttl.." "..mapheaderinfo[gamemap].zonttl
		end
		
		if clearscoreflag == true then
			--Wipe scores at the start of the first round
			COM_BufInsertText(server, "ATO_ClearScores")
			clearscoreflag = false
		end
		
		tIntCounter = CV_FindVar("inttime").value * TICRATE --Reset intermission timer
	end
	
	if aTourney.tourneyInSet then
		--Monitor to see if a disconnected player returned yet.
		for p in players.iterate do
			if p.valid and savedScores[p.name] then
				print("\x85**RECONNECTION** "..p.name.." has reconnected. Assigning them their saved score of \x83"..tostring(savedScores[p.name]))
				p.score = savedScores[p.name]
				savedScores[p.name] = nil
			end
		end
	end
	
	roundstatus = RS_THINK
end
addHook("ThinkFrame", ThinkFrame)

local function IntermissionThinker()
	roundstatus = RS_INT
	
	if aTourney.tourneyInSet and not didCommand then
		if not didAnnounce then -- print announcements
			didAnnounce = true
			
			if aTourney.raceNumber != 0 then
				PrintScores()
			end
			
			if aTourney.raceNumber < aTourney.totalMaps then
				aTourney.raceNumber = $ + 1
				
				if aTourney.isRandomSet then
					if aTourney.raceNumber >= aTourney.totalMaps then
						chatprint("\133<TOURNEY>\x83 Up Next: \x85LAST RACE OF THE SET!", true)
						lastRaceOfRandom = true
					elseif aTourney.raceNumber < aTourney.totalMaps then
						chatprint("\133<TOURNEY>\x83 Up Next: Race #"..tostring(aTourney.raceNumber).." of "..tostring(aTourney.totalMaps), true)	
					end
				else
					chatprint("\133<TOURNEY>\x83 Up Next: "..mapheaderinfo[tostring(aTourney.tourneyMapList[1])].lvlttl.." "..mapheaderinfo[tostring(aTourney.tourneyMapList[1])].zonttl, true)
					if aTourney.raceNumber >= aTourney.totalMaps then
						chatprint("\133<TOURNEY>\x85 LAST RACE OF THE SET!", true)
						lastRaceOfRandom = true --This variable name doesn't make sense anymore but nobody's reading this anyway...right?
					else
						chatprint("\133<TOURNEY>\x83 Race #"..tostring(aTourney.raceNumber).." of "..tostring(aTourney.totalMaps), true)	
					end				
				end
			elseif aTourney.raceNumber >= aTourney.totalMaps and not lastRaceOfRandom then
				chatprint("\133<TOURNEY>\x83 THE SET HAS CONCLUDED! Let's see who won!", true)
			end			
		end
		
		if tIntCounter > 15 then --give it a little buffer.
			tIntCounter = max(0, $-1)
			return --fuck off until then
		end
		
		if (aTourney.raceNumber < aTourney.totalMaps or lastRaceOfRandom) and not aTourney.isRandomSet then --Swap maps over, random sets let the game do it's thing
			COM_BufInsertText(server, "map "..tostring(aTourney.tourneyMapList[1]))
		elseif aTourney.raceNumber >= aTourney.totalMaps and not lastRaceOfRandom then
			if PauseOnSetEnd.value then
				tIntCounter = CV_FindVar("inttime").value * TICRATE
				aTourney.tourneyInSet = false
				savedScores = {}
				COM_BufInsertText(server, "pause")
			else
				aTourney.tourneyInSet = false
				savedScores = {}
			end	
		end
		
		didCommand = true		
	end
end
addHook("IntermissionThinker", IntermissionThinker)

local function VoteThinker()
	roundstatus = RS_VOTE --This used to do stuff but now it's just here to keep track
end
addHook("VoteThinker", VoteThinker)

local function PlayerQuit(p, r)
	--Save a disconnecting player's score
	if aTourney.tourneyInSet and not clearscoreflag then
		local dr = "Player Left"
		if r == KR_KICK then
			dr = "Player was kicked"
		elseif r == KR_PINGLIMIT
			dr = "Player broke pinglimit"
		elseif r == KR_SYNCH
			dr = "Player synchfailed"
		elseif r == KR_TIMEOUT
			dr = "Player timed out"
		elseif r == KR_BAN
			dr = "Player was banned"
		end
		print("\x85**DISCONNECTION** "..p.name.." disconnected in race number \x83"..tostring(aTourney.raceNumber).." \x85with a score of \x83"..tostring(p.score).." \x85 Leave reason - \x83"..dr)
		if p.score > 0 then
			savedScores[p.name] = p.score
		end		
	end
end
addHook("PlayerQuit", PlayerQuit)

--          Netsyncs
--Just sync everything, don't risk it when it comes to a tourney
addHook("NetVars", function(n)
	aTourney.tourneyMapList = n($)
	aTourney.raceNumber = n($)
	aTourney.totalMaps = n($)
	aTourney.tourneyInSet = n($)
	didAnnounce = n($)
	clearscoreflag = n($)
	roundstatus = n($)
	tIntCounter = n($)
	logMapBuffer = n($)
	savedScores = n($)
	aTourney.isRandomSet = n($)
	aTourney.bluetotalscore = n($)
	aTourney.orangetotalscore = n($)
	lastRaceOfRandom = n($)
	didCommand = n($)
end)

